==> sems-0.8.12.tar.gz: sems-0.8.12/plug-in/isdngw/IsdnDeviceIncoming.h <==
/*
 * $Id: IsdnDeviceIncoming.h,v 1.1.4.2 2004/01/23 13:40:58 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _ISDNDEVICEINCOMING_H_
#define _ISDNDEVICEINCOMING_H_

#include <pthread.h>    /* for mutex var */
#include "IsdnDevice.h" /* parent class */
#include "globals.h"    /* for FAIL */
#include "AmCmd.h"      /* for cmd in setCommand */

/** listening_state contains state of incoming devices */

/** isdn device is initialized and idle, no number is assigned */
// #define LISTENING_STATE_IDLE        0
/** isdn device waits for pstn call on one specified number (msn) */
// #define LISTENING_STATE_LISTENING   1
/** a call has arrived on assigned number, the caller hears ringing tone */
// #define LISTENING_STATE_RINGING     2
/** PSTN2SIP connection established, voice is being transmitted */
// #define LISTENING_STATE_CONNECTED   3

/** An incoming isdn device. 
 */
class IsdnDeviceIncoming : public IsdnDevice
{
private:
	char* m_incoming_number;   /* number to listen on (msn) */
	char m_call_id[100];       /* id to recognize self created onInvite (see AmApi) */
//	pthread_mutex_t mut_state; /* mutex to protect changes of device state */
	pthread_t t_listener;      /* listening thread for this isdn device */
	bool m_isListening; 
	bool m_isTerminated; 
	AmCmd m_cmd;               /* cmd object, needed for cancel on PSTN hangup */

	pthread_t* listenOnNumber(char *number); /* listen on specified number, creates thread */
	int acceptCallForKey(const char* key);	 /* threadsafe getter for listening state */
	
public:
	IsdnDeviceIncoming(char* devicename);

	void setCommand(AmCmd& cmd);
	
	void onBeforeCallAccept(AmRequest* req);
	void onError(unsigned int code, const string& reason);
	void onBye();
	
	void cleanup();

	const char* getType() { return "INCOMING"; };
	
	/* return a locked incoming isdn device */
	static IsdnDevice* getLockedDevice();

	int connectVoice();
	
	/* initialize incoming devices logic, must be called once prior to reassign */
	static void initializeOnce();
	
	/* (re)assigns threads and isdn devices to incoming numbers. threadsafe. */
	static void reassignIncomingDevices();

	/* returns the device the incoming call referenced by key belongs to */
	static IsdnDeviceIncoming* getPendingCallDevice(const char* key);
	
	/* call this method if ATA failed to unlock and reuse the device. */
	void resetFailedIncomingDevice(); 
	
	/* thread start function, simply calls listen() on right object */
	static void* _listen(void *);

	/* wait for incoming calls */
	void listen();

	void terminateListener();
};

extern pthread_mutex_t m_reass;  /* mutex to protect reassigning of incoming devices */

#endif

==> sems_2004_01_04.tar.gz: sems_2004_01_04/plug-in/isdngw/IsdnDeviceIncoming.h <==
/*
 * $Id: IsdnDeviceIncoming.h,v 1.1 2003/08/28 16:19:58 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _ISDNDEVICEINCOMING_H_
#define _ISDNDEVICEINCOMING_H_

#include <pthread.h>    /* for mutex var */
#include "IsdnDevice.h" /* parent class */
#include "globals.h"    /* for FAIL */
#include "AmCmd.h"      /* for cmd in setCommand */

/** listening_state contains state of incoming devices */

/** isdn device is initialized and idle, no number is assigned */
#define LISTENING_STATE_IDLE        0
/** isdn device waits for pstn call on one specified number (msn) */
#define LISTENING_STATE_LISTENING   1
/** a call has arrived on assigned number, the caller hears ringing tone */
#define LISTENING_STATE_RINGING     2
/** PSTN2SIP connection established, voice is being transmitted */
#define LISTENING_STATE_CONNECTED   3

/** An incoming isdn device. 
 */
class IsdnDeviceIncoming : public IsdnDevice
{
private:
	char* m_incoming_number;   /* number to listen on (msn) */
	char m_call_id[100];       /* id to recognize self created onInvite (see AmApi) */
	pthread_mutex_t mut_state; /* mutex to protect changes of device state */
	pthread_t t_listener;      /* listening thread for this isdn device */
	int m_listening_state;     /* mut_state protected listening state */
	AmCmd m_cmd;               /* cmd object, needed for cancel on PSTN hangup */

	pthread_t* listenOnNumber(char *number); /* listen on specified number, creates thread */
	int acceptCallForKey(const char* key);	 /* threadsafe getter for listening state */
	
public:
	IsdnDeviceIncoming(char* devicename);

	void setCommand(AmCmd& cmd);
	
	void onBeforeCallAccept(AmRequest* req);
	void onError(unsigned int code, const string& reason);
	void onBye();
	
	void cleanup();

	const char* getType() { return "INCOMING"; };
	
	/* return a locked incoming isdn device */
	static IsdnDevice* getLockedDevice();

	int connectVoice();
	
	/* initialize incoming devices logic, must be called once prior to reassign */
	static void initializeOnce();
	
	/* (re)assigns threads and isdn devices to incoming numbers. threadsafe. */
	static void reassignIncomingDevices();

	/* returns the device the incoming call referenced by key belongs to */
	static IsdnDeviceIncoming* getPendingCallDevice(const char* key);
	
	/* call this method if ATA failed to unlock and reuse the device. */
	void resetFailedIncomingDevice(); 
	
	/* thread start function, simply calls listen() on right object */
	static void* _listen(void *);

	/* wait for incoming calls */
	void listen();
};

extern pthread_mutex_t m_reass;  /* mutex to protect reassigning of incoming devices */

#endif
